package us.kostenko.gps_service_plugin

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.SharedPreferences
import android.os.Build
import android.util.Log
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import us.kostenko.gps_service_plugin.utils.customPrefs
import us.kostenko.gps_service_plugin.utils.set

class GpsServicePlugin(
        private val context: Context,
        private val activity: Activity?
) : MethodCallHandler {

    companion object {

        @JvmStatic
        val TAG = GpsServicePlugin::class.java.simpleName

        @JvmStatic
        val SHARED_PREFERENCES_KEY = "geofencing_plugin_cache"

        @JvmStatic
        val CALLBACK_HANDLE_KEY = "callback_handle"

        @JvmStatic
        val CALLBACK_DISPATCHER_HANDLE_KEY = "callback_dispatch_handle"

        @JvmStatic
        private val REQUIRED_PERMISSIONS = arrayOf(Manifest.permission.ACCESS_FINE_LOCATION)

        @JvmStatic
        private val PERMISSIONS_REQUEST_CODE = 12312

        @JvmStatic
        private var locationHandler: LocationHandler? = null

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val plugin = GpsServicePlugin(registrar.context(), registrar.activity())
            val channel = MethodChannel(registrar.messenger(), "us.kostenko/gps_plugin")
            channel.setMethodCallHandler(plugin)
        }

        @JvmStatic
        private fun initializeService(context: Context, args: Any) {
            val map = args as? Map<*, *>
            Log.d(TAG, "Initializing GeofencingService")
            val callbackDispatcherHandle = map?.get("callbackDispatcherHandle") as? Long ?: 0L
            val callbackHandle = map?.get("callbackHandle") as? Long ?: 0L

            Log.d(TAG,"GpsServicePlugin, callbackHandle: $callbackHandle")
            val prefs: SharedPreferences = context.customPrefs(SHARED_PREFERENCES_KEY)
            prefs[CALLBACK_DISPATCHER_HANDLE_KEY] = callbackDispatcherHandle
            prefs[CALLBACK_HANDLE_KEY] = callbackHandle

            locationHandler = LocationHandler(context.applicationContext)
        }

        @JvmStatic
        private fun startListeningLocation(result: Result?) {
            Log.d(TAG, "startListeningLocation")
            // start service
            locationHandler?.start()
            result?.success(true)
        }

        @JvmStatic
        private fun stopListeningLocation(result: Result?) {
            Log.d(TAG, "stopListeningLocation")
            // stop service
            locationHandler?.stop()
            result?.success(true)
        }

        @JvmStatic
        private fun dispose(result: Result?) {
            Log.d(TAG, "dispose")
            // dispose service
            locationHandler?.dispose()
            if (locationHandler?.isTreckingLocation == false) locationHandler = null
            result?.success(true)
        }

        @JvmStatic
        private fun isTreckingLocation(result: Result?) {
            val isTrecking = locationHandler?.isTreckingLocation ?: false
            Log.d(TAG, "isTreckingLocation: $isTrecking")
            result?.success(isTrecking)
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "GpsManager.initializeService" -> {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                    activity?.requestPermissions(REQUIRED_PERMISSIONS, PERMISSIONS_REQUEST_CODE)
                }
                initializeService(context, call.arguments)
                result.success(true)
            }
            "GpsManager.startListeningLocation" -> startListeningLocation(result)
            "GpsManager.stopListeningLocation" -> stopListeningLocation(result)
            "GpsManager.isTreckingLocation" -> isTreckingLocation(result)
            "GpsManager.dispose" -> dispose(result)
            else -> result.notImplemented()
        }
    }
}
