package us.kostenko.gps_service_plugin

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.Build
import android.os.IBinder
import android.util.Log
import androidx.lifecycle.LifecycleObserver
import us.kostenko.gps_service_plugin.LocationService.Companion.IS_TRECKING_STARTED
import us.kostenko.gps_service_plugin.utils.defaultPrefs
import us.kostenko.gps_service_plugin.utils.get

class LocationHandler(
        private val context: Context
): LifecycleObserver {

    private val serviceIntent: Intent by lazy { 
        Intent(context, LocationService::class.java)
    }
    private var locationService: LocationService? = null

    private val serviceConnection = object : ServiceConnection {

        override fun onServiceConnected(className: ComponentName, service: IBinder) {
            val name = className.className
            if (name.endsWith(LocationService::class.java.simpleName)) {
                locationService = (service as LocationService.LocationServiceBinder).service
            }
        }

        override fun onServiceDisconnected(className: ComponentName) {
            if (className.className == "BackgroundService") {
                locationService = null
            }
        }
    }

    val isTreckingLocation: Boolean
        get() = context.defaultPrefs[IS_TRECKING_STARTED] ?: false

    init {
        startService()
    }

    fun start() {
        // start session
        if (!isTreckingLocation) {
            Log.d(TAG, "startingSession")
            locationService?.startSession()
        }
    }

    fun stop() {
        // stop service
        if (isTreckingLocation) {
            Log.d(TAG, "stoppingSession")
            locationService?.stopSession()
        }
    }

    fun dispose() {
        context.unbindService(serviceConnection)
        if (!isTreckingLocation) locationService?.stopService(serviceIntent)
    }

    private fun startService() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            context.startForegroundService(serviceIntent)
        } else {
            context.startService(serviceIntent)
        }
        context.bindService(serviceIntent, serviceConnection, Context.BIND_AUTO_CREATE)
    }

    companion object {
        val TAG = LocationHandler::class.java.simpleName
    }
}