package us.kostenko.gps_service_plugin

import android.content.Context
import android.content.pm.PackageManager
import android.os.Looper
import androidx.core.app.ActivityCompat
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationCallback
import com.google.android.gms.location.LocationRequest
import com.google.android.gms.location.LocationResult
import com.google.android.gms.location.LocationServices

class LocationListener(
        private val context: Context,
        private val onLocationResult: (LocationResult?) -> Unit,
        private val priority: Int? = null,
        private val interval: Long? = null,
        private val fastestInterval: Long? = null,
        private val smallestDisplacement: Float? = null,
        private val maxWaitTime: Long? = null
) {

    private val fusedLocationClient: FusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(context)

    // Create the location request to startListening receiving updates
    private val locationRequest = LocationRequest().apply {
        priority = this@LocationListener.priority ?: LocationRequest.PRIORITY_HIGH_ACCURACY
        interval = this@LocationListener.interval ?: UPDATES_INTERVAL
        fastestInterval = this@LocationListener.fastestInterval ?: FASTEST_INTERVAL
//        smallestDisplacement = this@LocationListener.smallestDisplacement ?: SMALLEST_DISPLACEMENT
        maxWaitTime = this@LocationListener.maxWaitTime ?: MAX_WAIT_TIME
    }

    private val locationCallback: LocationCallback = object : LocationCallback() {
        override fun onLocationResult(locationResult: LocationResult?) {
            this@LocationListener.onLocationResult(locationResult)
        }
    }

    fun startTrecking() {
        val permissionCheckResult = ActivityCompat.checkSelfPermission(context, android.Manifest.permission.ACCESS_FINE_LOCATION)
        if (permissionCheckResult != PackageManager.PERMISSION_GRANTED) return

        fusedLocationClient.requestLocationUpdates(
                locationRequest, locationCallback, Looper.myLooper()
        )
    }

    fun stopTracking() {
        fusedLocationClient.removeLocationUpdates(locationCallback)
    }

    companion object {

        val TAG: String = LocationListener::class.java.simpleName

        const val UPDATES_INTERVAL = 10 * 1000L            // desired interval for active location updates, in milliseconds.
        const val FASTEST_INTERVAL = 5 * 1000L            // the fastest interval for location updates, in milliseconds.
        const val SMALLEST_DISPLACEMENT = 1f                    // minimum displacement between location updates in meters
        const val MAX_WAIT_TIME = UPDATES_INTERVAL * 3    // If you pass a value at least 2x larger than the interval specified with setInterval(long), then location delivery may be delayed and multiple locations can be delivered at once.
    }
}