import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

import 'gps_manager.dart';

void callbackDispatcher() {
  // 1. Initialize MethodChannel used to communicate with the platform portion of the plugin.
  const MethodChannel _backgroundChannel =
      MethodChannel('us.kostenko/gps_plugin_background');

  // 2. Setup internal state needed for MethodChannels.
  WidgetsFlutterBinding.ensureInitialized();

  // 3. Listen for background events from the platform portion of the plugin.
  _backgroundChannel.setMethodCallHandler((MethodCall call) async {
    final Map<dynamic, dynamic> args = call.arguments;

    // 3.1. Retrieve callback instance from handle.
    final callbackHandle = args["callbackHandle"];
    print("callbackDispatcher, callback handle: $callbackHandle");
    final GpsCallback callback = PluginUtilities.getCallbackFromHandle(
      CallbackHandle.fromRawHandle(callbackHandle)
    );
    assert(callback != null);

    if ((args["type"] as int) == 0) { // it's position
      final map = args["locationMap"];
	    final pos = Position(
		    time: map["time"],
		    lat: map["latitude"],
		    lon: map["longitude"],
		    accuracy: map["accuracy"],
		    altitude: map["altitude"],
		    speed: map["speed"],
	    );
      callback(pos: pos);
    } else {
      final msg = args["event"].toString();
      callback(msg: msg);
    }
  });

  // 4. Alert plugin that the callback handler is ready for events.
  _backgroundChannel.invokeMethod('LocationService.initialized');
}
