import 'dart:ui';

import 'package:flutter/services.dart';
import 'package:gps_service_plugin/callback_dispatcher.dart';

typedef GpsCallback = void Function({Position pos, String msg});

class Position {
	String time;
	double lat;
	double lon;
	double accuracy;
	double altitude;
	double speed;
	
	Position({
		this.time,
		this.lat,
		this.lon,
		this.accuracy,
		this.altitude,
		this.speed
	});
	
	String toString() {
		return "time: $time, lat: $lat, lon: $lon, accuracy: $accuracy, altitude: $altitude, speed: $speed";
	}
	
	Map<String, dynamic> toMap() {
		return {
			"time": this.time,
			"latitude": this.lat,
			"longitude": this.lon,
			"accuracy": this.accuracy,
			"altitude": this.altitude,
			"speed": this.speed,
		};
	}
}

class GpsManager {
	static const MethodChannel _channel = MethodChannel('us.kostenko/gps_plugin');
	
	/// Initialize the plugin and request relevant permissions from the user.
	static Future<void> initialize(GpsCallback callback) async {
		final CallbackHandle callbackDispatcherHandle = PluginUtilities.getCallbackHandle(callbackDispatcher);
		final CallbackHandle callbackHandle = PluginUtilities.getCallbackHandle(callback);
		print("GpsManager, callback handle: $callbackHandle");
		final args = {
			"callbackDispatcherHandle": callbackDispatcherHandle.toRawHandle(),
			"callbackHandle": callbackHandle.toRawHandle(),
		};
		await _channel.invokeMethod(
				'GpsManager.initializeService',
				args
		);
	}
	
	static Future<void> startListeningLocation() async {
		await _channel.invokeMethod(
				'GpsManager.startListeningLocation',
		);
	}
	
	static Future<void> stopListeningLocation() async {
		await _channel.invokeMethod(
				'GpsManager.stopListeningLocation',
		);
	}
	
	static Future<bool> isTreckingLocation() async {
		return await _channel.invokeMethod(
				'GpsManager.isTreckingLocation',
		);
	}
	
	static Future<void> dispose() async {
		await _channel.invokeMethod(
				'GpsManager.dispose',
		);
	}
}